<?php

namespace Dac\Client;

use Dac\Exception\ConfigException;
use Dac\Exception\DACException;
use Dac\Model\Identity;
use Dac\Model\ResponseData;
use Dac\Utils\Config;
use Dac\Utils\Message;
use Dac\Utils\Tools;


class DACClient
{
    public Config $config;
    public HttpsClient $httpsClient;

    const SharedSubmissionPath = '/v1/%s/sharedstore/message/submission';
    const SharedQueryMessagePath = '/v1/%s/sharedstore/message/query';
    const SharedResourcePath     = "/v1/%s/sharedstore/message/resource";

    const AsyncBatchMintSplite = 100;
    const AsyncBatchTransferSplite = 30;


    /**
     * @param $configPath
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function __construct($configPath)
    {
        $this->config = new Config($configPath);
        $this->httpsClient = new HttpsClient($this->config->getTlsConfig());
    }

    /**
     * @param $funcName
     * @param $args
     * @return ResponseData|false
     * @throws ConfigException
     * @throws DACException
     * @throws \Dac\Exception\NFTException
     */
    public function sendSubmissionRequest($funcName, $args)
    {
        if ($this->config->getServiceMode() == Config::SharedMode) {
            return $this->sendSubmissionRequest2Shared($funcName, $args);
        } else {
            throw new ConfigException("not support special mode yet");
        }
    }

    /**
     * @param $funcName
     * @param $args
     * @return ResponseData|false
     * @throws ConfigException
     * @throws DACException
     * @throws \Dac\Exception\NFTException
     */
    public function sendQueryRequest($funcName, $args)
    {
        if ($this->config->getServiceMode() == Config::SharedMode) {
            return $this->sendQueryRequest2Shared($funcName, $args);
        } else {
            throw new ConfigException("not support special mode yet");
        }
    }

    /**
     * @param $funcName
     * @param $args
     * @return ResponseData|false
     * @throws DACException
     * @throws \Dac\Exception\NFTException
     */
    private function sendSubmissionRequest2Shared($funcName, $args)
    {
        return $this->sendRequest2Shared($funcName, $args, self::SharedSubmissionPath);
    }


    /**
     * @param $funcName
     * @param $args
     * @return ResponseData|false
     * @throws DACException
     * @throws \Dac\Exception\NFTException
     */
    private function sendQueryRequest2Shared($funcName, $args)
    {
        return $this->sendRequest2Shared($funcName, $args, self::SharedQueryMessagePath);
    }

    /**
     * @param $funcName
     * @param $args
     * @param $path
     * @return ResponseData|false
     * @throws DACException
     * @throws \Dac\Exception\NFTException
     */
    private function sendRequest2Shared($funcName, $args, $path)
    {
        $url = $this->config->getService()['url'] . sprintf($path, $this->config->getProjectID());
        $msg = new Message($this->config);
        $preMsg = $msg->buildPreparationMsg($funcName, $args);
        $result = $this->httpsClient->doPostRequest($url, [], $preMsg->serializeToString());
        if ($result['errCode'] != HttpsClient::OK) {
            throw new DACException($result['errMsg']);
        }
        $resData = new ResponseData("", "");
        if (!empty($result['response'])) {
            $resData = Tools::parseResponseRawData($result['response'][0]);
        }
        return $resData;
    }
    public function sendResourceSharedRequest($funcName, $args, $header)
    {
        $url = $this->config->getService()['url'] . sprintf(self::SharedResourcePath, $this->config->getProjectID());
        $msg = new Message($this->config);
        $preMsg = $msg->buildPreparationMsg($funcName, $args);
        $result = $this->httpsClient->doResourcePostRequest($url, $header, $preMsg->serializeToString());
        if ($result['errCode'] != HttpsClient::OK) {
            throw new DACException("request fail".$result['errMsg']);
        }
        return $result['response'];
    }
    /**
     * @param string $cert
     * @param string $key
     * @return void
     */
    public function switchIdentityWithPem(string $cert, string $key)
    {
        $this->config->updateIdentity($cert, $key);
    }


    /**
     * @param $funcName
     * @param $collectionIndex
     * @param int $startTokenID
     * @param int $amount
     * @param string $tokenURI
     * @return ResponseData|false
     * @throws ConfigException
     * @throws DACException
     * @throws \Dac\Exception\NFTException
     */
    public function asyncSubmissionBatchMintRequest($funcName, $collectionIndex, int $startTokenID, int $amount, string $tokenURI): bool|ResponseData
    {
        if ($this->config->getServiceMode() == Config::SharedMode) {
            return $this->asyncSubmissionBatchMintRequset2Shared($funcName, $collectionIndex, $startTokenID, $amount, $tokenURI);
        } else {
            throw new ConfigException("not support special mode yet");
        }
    }

    /**
     * @param string $funcName
     * @param string $from
     * @param string $to
     * @param array $ids
     * @return bool|ResponseData
     * @throws ConfigException
     * @throws DACException
     */
    public function asyncSubmissionBatchTransferRequest(string $funcName, string $from, string $to, array $ids): bool|ResponseData
    {
        if ($this->config->getServiceMode() == Config::SharedMode) {
            return $this->asyncSubmissionBatchTransferRequest2Shared($funcName, $from, $to, $ids);
        } else {
            throw new ConfigException("not support special mode yet");
        }
    }

    public function asyncSubmissionBatchTransferRequest2Shared(string $funcName, string $from, string $to, array $ids): bool|ResponseData
    {
        $nums = $this->computeAsyncNums(count($ids), self::AsyncBatchTransferSplite);
        $unique = hash('sha256', $from . $to . time() . rand());
        $header = ["BatchOperationID" => $unique];

        $msgPackage = [];
        for ($k = 0; $k < $nums; $k++) {
            $eachArgs = [];
            $eachArgs[0] = $from;
            $eachArgs[1] = $to;
            if ($k == $nums - 1) {
                $eachArgs[2] = json_encode(array_slice($ids, self::AsyncBatchMintSplite * ($nums - 1)));
            } else {
                $eachArgs[2] = json_encode(array_slice($ids, self::AsyncBatchMintSplite * $k, self::AsyncBatchMintSplite * ($k + 1)));
            }

            $msg = new Message($this->config);
            $rawMsg = $msg->buildPreparationMsg($funcName, $eachArgs);
            $msgPackage[] = Tools::strToBytes($rawMsg->serializeToString());
        }
        $url = $this->config->getService()['url'] . sprintf(self::SharedSubmissionPath, $this->config->getProjectID());
        $result = $this->httpsClient->doPostRequest($url, $header, json_encode($msgPackage, JSON_INVALID_UTF8_IGNORE));
        if ($result['errCode'] != HttpsClient::OK) {
            throw new DACException($result['errMsg']);
        }
        $resData = new ResponseData("", "");
        if (!empty($result['response'])) {
            $resData = Tools::parseResponseRawData($result['response'][0]);
        }
        return $resData;

    }

    /**
     * @param $funcName
     * @param $collectionIndex
     * @param int $startTokenID
     * @param int $amount
     * @param string $tokenURI
     * @return ResponseData|false
     * @throws DACException
     * @throws \Dac\Exception\NFTException
     */
    private function asyncSubmissionBatchMintRequset2Shared($funcName, $collectionIndex, int $startTokenID, int $amount, string $tokenURI)
    {
        $nums = $this->computeAsyncNums($amount, self::AsyncBatchMintSplite);
        $unique = hash('sha256', $collectionIndex . $startTokenID . $amount . time());
        $header = ["BatchOperationID" => $unique];

        $msgPackage = [];
        for ($k = 0; $k < $nums; $k++) {
            $tmpAmount = self::AsyncBatchMintSplite;
            if ($k == $nums - 1) {
                $tmpAmount = $amount - self::AsyncBatchMintSplite * ($nums - 1);
            }
            $msg = new Message($this->config);
            $rawMsg = $msg->buildPreparationMsg($funcName, array($collectionIndex, $startTokenID, $tmpAmount, $tokenURI));
            $msgPackage[] = Tools::strToBytes($rawMsg->serializeToString());
            $startTokenID = $startTokenID + self::AsyncBatchMintSplite;
        }
        $url = $this->config->getService()['url'] . sprintf(self::SharedSubmissionPath, $this->config->getProjectID());
        $result = $this->httpsClient->doPostRequest($url, $header, json_encode($msgPackage, JSON_INVALID_UTF8_IGNORE));
        if ($result['errCode'] != HttpsClient::OK) {
            throw new DACException($result['errMsg']);
        }
        $resData = new ResponseData("", "");
        if (!empty($result['response'])) {
            $resData = Tools::parseResponseRawData($result['response'][0]);
        }
        return $resData;
    }

    private function computeAsyncNums($amount, int $splite)
    {
        $count = $amount / $splite;
        return ceil($count);
    }

    /**
     * @param string $identityName
     * @return void
     * @throws ConfigException
     * @throws DACException
     */
    public function switchIdentityWithName(string $identityName)
    {
        $identityConfig = $this->config->getIdentityByName($identityName);
        $identity = new Identity();
        $identity->loadWithPath($identityConfig['certPath'], $identityConfig['keyPath']);
        $this->config->setIdentity($identity);
    }


}