<?php

namespace Dac\Manager;

use Dac\Exception\NFTException;
use Dac\Model\BatchMintData;
use Dac\Model\ID;
use Dac\Utils\Tools;

class NFT1155Manager extends BaseManager
{
    const MaxBatchQuery = 200;
    const MaxBatchRequest = 5000;
    const MinBatchRequest = 1;
    const BatchMint = "batchMint";
    const SafeBatchTransferFrom = "safeBatchTransferFrom";
    const SafeTransferFrom = "safeTransferFrom";
    const SetApprovalForAll = "setApprovalForAll";
    const IsApprovedForAll = "isApprovedForAll";
    const BalanceOfBatch = "balanceOfBatch";
    const OwnerOf = "ownerOf";
    const URI = "uri";

    public function batchMint(string $collectionIndex, BatchMintData $batchMintData)
    {
        if (empty($collectionIndex) || !$batchMintData->isValid()) {
            throw new NFTException("collection index, startTokenID, amount or tokenURI is empty");
        }
        $amount = $batchMintData->getAmount();
        $startToken = $batchMintData->getStartTokenID();

        if ($amount > self::MaxBatchRequest || $amount < self::MinBatchRequest) {
            throw new NFTException("amount should in [1,5000]");
        }
        if ($amount - 1 > PHP_INT_MAX - $startToken) {
            throw new NFTException("can't mint excess of NFTs due to greater than the PHP_INT_MAX");
        }
        $resp = $this->dacClient->asyncSubmissionBatchMintRequest(self::BatchMint, $collectionIndex, $startToken, $amount, $batchMintData->tokenURI->toString());
        return $resp->getTxid();
    }

    /**
     * @param string $from
     * @param string $to
     * @param array $ids
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function safeBatchTransferFrom(string $from, string $to, array $ids): string
    {
        if (empty($from) || empty($to) || count($ids) == 0) {
            throw new NFTException("from, to, collection Indexs, or ids is empty");
        }
        if (count($ids) > self::MaxBatchRequest) {
            throw new NFTException("ids array size should in [1,5000]");
        }
        $idsStr = [];
        foreach ($ids as $k => $v) {
            $idsStr[] = $v->getId();
        }
        $resp = $this->dacClient->asyncSubmissionBatchTransferRequest(self::SafeBatchTransferFrom, $from, $to, $idsStr);
        return $resp->getTxid();
    }

    /**
     * @param string $from
     * @param string $to
     * @param ID $id
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function safeTransferFrom(string $from, string $to, ID $id): string
    {
        if (empty($from) || empty($to) || empty($id->getId())) {
            throw new NFTException("from, to or id string is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::SafeTransferFrom, [$from, $to, $id->getId()]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $owner
     * @param string $operator
     * @param boolean $isApproved
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function setApprovalForAll(string $owner, string $operator, bool $isApproved)
    {
        if (empty($owner) || empty($operator)) {
            throw new NFTException("owner or operator is empty");
        }
        $approvedStr = $isApproved ? 'true' : 'false';
        $respData = $this->dacClient->sendSubmissionRequest(self::SetApprovalForAll, [$owner, $operator, $approvedStr]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $owner
     * @param string $operator
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function isApprovedForAll(string $owner, string $operator)
    {
        if (empty($owner) || empty($operator)) {
            throw new NFTException("owner or operator is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::IsApprovedForAll, [$owner, $operator]);
        return json_decode($respData->getData());
    }

    /**
     * @param array $identites
     * @param array $ids
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function balanceOfBatch(array $identites, array $ids)
    {
        if (count($identites) == 0 || count($ids) == 0) {
            throw new NFTException("identities or ids is empty");
        }
        if (count($identites) > self::MaxBatchQuery) {
            throw new NFTException("the number of queries exceeds the maximum 200");
        }
        $idsStr = [];
        foreach ($ids as $k => $v) {
            $idsStr[] = $v->getId();
        }
        $idenReq = json_encode($identites);
        $idReq = json_encode($idsStr);
        if (empty($idReq) || empty($idenReq)) {
            throw new NFTException("identities or ids is error");
        }
        $respData = $this->dacClient->sendQueryRequest(self::BalanceOfBatch, [$idenReq, $idReq]);
        return json_decode($respData->getData());
    }

    /**
     * @param ID $id
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function ownerOf(ID $id)
    {
        if (empty($id->getId())) {
            throw new NFTException("id string is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::OwnerOf, [$id->getId()]);
        return json_decode($respData->getData());
    }

    /**
     * @param ID $id
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function uri(ID $id)
    {
        if (empty($id->getId())) {
            throw new NFTException("id string is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::URI, [$id->getId()]);
        return json_decode($respData->getData());
    }

}