<?php

namespace Dac\Manager;

use Dac\Exception\DACException;
use Dac\Exception\NFTException;
use Dac\Model\Metadata;
use Dac\Model\MintResp;
use Dac\Model\NFT;
use Dac\Utils\Tools;

class NFT721Manager extends BaseManager
{
    const Mint = "mint";
    const Burn = "burn";

    const SupervisorBurn = "supervisorBurn";
    const TokenURI = "tokenURI";
    const OwnerOf = "ownerOf";
    const TransferFrom = "transferFrom";
    const Approve = "approve";
    const GetApproved = "getApproved";
    const IsApprovedForAll = "isApprovedForAll";
    const SetApprovedForAll = "setApprovalForAll";
    const Name = "name";
    const Symbol = "symbol";
    const SetOption = "setOption";
    const IsMinted = "isMinted";
    const CreateNFTCollection = "createNFTCollection";
    const SetCollectionLimit = "setCollectionLimit";
    const Collection = "collections";

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @param Metadata $tokenURI
     * @return MintResp
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function mint(string $collectionIndex, string $tokenId, Metadata $tokenURI)
    {
        if (empty($collectionIndex) || $tokenId == "" || empty($tokenURI)) {
            throw new NFTException("collection index, tokenId or tokenURI is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::Mint, [$collectionIndex, $tokenId, $tokenURI->toString()]);
        $stdClass = json_decode($respData->getData(), false);
        $nft = new NFT($stdClass);
        return new MintResp($nft, $respData->getTxid());
    }


    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function burn(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index or tokenId is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::Burn, [$collectionIndex, $tokenId]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function supervisorBurn(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index or tokenId is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::SupervisorBurn, [$collectionIndex, $tokenId]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return Metadata
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function tokenURI(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index or tokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::TokenURI, [$collectionIndex, $tokenId]);
        $arr = json_decode($respData->getData(), true);
        $md = new Metadata();
        $md->loadFromArray($arr);
        return $md;
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return mixed
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function ownerOf(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index or tokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::OwnerOf, [$collectionIndex, $tokenId]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @param string $from
     * @param string $to
     * @param string $tokenId
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function transferFrom(string $collectionIndex, string $from, string $to, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "" || empty($from) || empty($to)) {
            throw new NFTException("collection index, from, to or tokenId is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::TransferFrom, [$collectionIndex, $from, $to, $tokenId]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @param string $operator
     * @param string $tokenId
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function approve(string $collectionIndex, string $operator, string $tokenId)
    {
        if (empty($collectionIndex) || empty($operator) || $tokenId == "") {
            throw new NFTException("collection index, operator or tokenId is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::Approve, [$collectionIndex, $operator, $tokenId]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return mixed
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function getApproved(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index, tokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::GetApproved, [$collectionIndex, $tokenId]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @param string $owner
     * @param string $operator
     * @param bool $isApproved
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function setApprovalForAll(string $collectionIndex, string $owner, string $operator, bool $isApproved)
    {
        if (empty($collectionIndex) || empty($operator) || empty($owner)) {
            throw new NFTException("collection index, owner or operator is empty");
        }
        $isApprovedStr = $isApproved ? 'true' : 'false';
        $respData = $this->dacClient->sendSubmissionRequest(self::SetApprovedForAll, [$collectionIndex, $owner, $operator, $isApprovedStr]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @param string $owner
     * @param string $operator
     * @return mixed
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function isAppravedForAll(string $collectionIndex, string $owner, string $operator)
    {
        if (empty($collectionIndex) || empty($operator) || empty($owner)) {
            throw new NFTException("collection index, owner or operator is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::IsApprovedForAll, [$collectionIndex, $owner, $operator]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return mixed
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function isMinted(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index or tokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::IsMinted, [$collectionIndex, $tokenId]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function createNFTCollection(string $collectionIndex)
    {
        if (empty($collectionIndex)) {
            throw new NFTException("collection index is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::CreateNFTCollection, [$collectionIndex, ""]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @return mixed
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function collections()
    {
        $respData = $this->dacClient->sendQueryRequest(self::Collection, []);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @param string $name
     * @param string $symbol
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function setOption(string $collectionIndex, string $name, string $symbol)
    {
        if (empty($collectionIndex) || empty($name) || empty($symbol)) {
            throw new NFTException("collection index, name or symbol is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::SetOption, [$collectionIndex, $name, $symbol]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @param int $times
     * @param int $duration
     * @return string
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function setCollectionLimit(string $collectionIndex, int $times, int $duration)
    {
        if (empty($collectionIndex)) {
            throw new NFTException("collection index is empty");
        }

        if ($times < 0 || $times > 1000 || $duration < 0 || $duration > 1000) {
            throw new NFTException("times and duration should in [0,1000]");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::SetCollectionLimit, [$collectionIndex, $times, $duration]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @return mixed
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function name(string $collectionIndex)
    {
        if (empty($collectionIndex)) {
            throw new NFTException("collection index is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::Name, [$collectionIndex]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @return mixed
     * @throws DACException
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     */
    public function symbol(string $collectionIndex)
    {
        if (empty($collectionIndex)) {
            throw new NFTException("collection index is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::Symbol, [$collectionIndex]);
        return json_decode($respData->getData());
    }

}