<?php

namespace Dac\Manager;

use Dac\Exception\NFTException;
use Dac\Model\Metadata;
use Dac\Model\MintResp;
use Dac\Model\NFT;
use Dac\Model\ParentInfo;
use Dac\Utils\Tools;

class NFT998Manager extends BaseManager
{
    const MintComposableAsset   = "mintComposableAsset";
    const GetChild              = "getChild";
    const TransferChild         = "transferChild";
    const SafeTransferChild     = "safeTransferChild";
    const RootOwnerOf           = "rootOwnerOf";
    const RootOwnerOfChild      = "rootOwnerOfChild";
    const OwnerOfChild          = "ownerOfChild";
    const TotalChildCollections = "totalChildCollections";
    const ChildContractByIndex  = "childContractByIndex";
    const TotalChildTokens      = "totalChildTokens";
    const ChildTokenByIndex     = "childTokenByIndex";

    const ERC998MagicValueLength = 24;

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @param Metadata $tokenURI
     * @return MintResp
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function mint(string $collectionIndex, string $tokenId, Metadata $tokenURI)
    {
        if (empty($collectionIndex) || $tokenId == "" || empty($tokenURI)) {
            throw new NFTException("collection index, tokenId or tokenURI is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::MintComposableAsset, [$collectionIndex, $tokenId, $tokenURI->toString()]);
        $stdClass = json_decode($respData->getData(), false);
        $nft = new NFT($stdClass);
        return new MintResp($nft, $respData->getTxid());
    }

    /**
     * @param string $from
     * @param string $toCollectionIndex
     * @param string $toTokenId
     * @param string $childCollectionIndex
     * @param string $childTokenId
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function getChild(string $from, string $toCollectionIndex, string $toTokenId, string $childCollectionIndex, string $childTokenId)
    {
        if (empty($from) || empty($toCollectionIndex) || $toTokenId == "" || empty($childCollectionIndex) || $childTokenId == "") {
            throw new NFTException("from, toCollection index, toTokenId, childCollection index or childTokenId is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::GetChild, [$from, $toCollectionIndex, $toTokenId, $childCollectionIndex, $childTokenId]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $fromCollectionIndex
     * @param string $fromTokenId
     * @param string $to
     * @param string $childCollectionIndex
     * @param string $childTokenId
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function transferChild(string $fromCollectionIndex, string $fromTokenId, string $to, string $childCollectionIndex, string $childTokenId)
    {
        if (empty($fromCollectionIndex) || $fromTokenId == "" || empty($to) || empty($childCollectionIndex) || $childTokenId == "") {
            throw new NFTException("fromCollection index, fromTokenId, to, childCollection index or childTokenId is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::TransferChild, [$fromCollectionIndex, $fromTokenId, $to, $childCollectionIndex, $childTokenId]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $fromCollectionIndex
     * @param string $fromTokenId
     * @param string $toCollectionIndex
     * @param string $childCollectionIndex
     * @param string $childTokenId
     * @param string $toTokenId
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function safeTransferChild(string $fromCollectionIndex, string $fromTokenId, string $toCollectionIndex, string $childCollectionIndex, string $childTokenId, string $toTokenId)
    {
        if (empty($fromCollectionIndex) || $fromTokenId == "" || empty($toCollectionIndex) || empty($childCollectionIndex) || $childTokenId == "" || $toTokenId == "") {
            throw new NFTException("fromCollection index, fromTokenId, toChildCollection index, childCollection index, childTokenId or toTokenId is empty");
        }
        $respData = $this->dacClient->sendSubmissionRequest(self::SafeTransferChild, [$fromCollectionIndex, $fromTokenId, $toCollectionIndex, $childCollectionIndex, $childTokenId, $toTokenId]);
        Tools::checkResponse($respData->getData());
        return $respData->getTxid();
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function rootOwnerOf(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index or toTokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::RootOwnerOf, [$collectionIndex, $tokenId]);
        return substr(json_decode($respData->getData()), self::ERC998MagicValueLength);
    }

    /**
     * @param string $childCollectionIndex
     * @param string $childTokenId
     * @return string
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function rootOwnerOfChild(string $childCollectionIndex, string $childTokenId)
    {
        if (empty($childCollectionIndex) || $childTokenId == "") {
            throw new NFTException("childCollection index or childTokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::RootOwnerOfChild, [$childCollectionIndex, $childTokenId]);
        return substr(json_decode($respData->getData()), self::ERC998MagicValueLength);
    }

    /**
     * @param string $childCollectionIndex
     * @param string $childTokenId
     * @return ParentInfo
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function ownerOfChild(string $childCollectionIndex, string $childTokenId)
    {
        if (empty($childCollectionIndex) || $childTokenId == "") {
            throw new NFTException("childCollection index or childTokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::OwnerOfChild, [$childCollectionIndex, $childTokenId]);
        $stdClass = json_decode($respData->getData(), false);
        return new ParentInfo($stdClass);
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function totalChildCollections(string $collectionIndex, string $tokenId)
    {
        if (empty($collectionIndex) || $tokenId == "") {
            throw new NFTException("collection index or tokenId is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::TotalChildCollections, [$collectionIndex, $tokenId]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @param string $index
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function childContractByIndex(string $collectionIndex, string $tokenId, string $index)
    {
        if (empty($collectionIndex) || $tokenId == "" || $index == "") {
            throw new NFTException("collection index, tokenId and or index is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::ChildContractByIndex, [$collectionIndex, $tokenId, $index]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @param string $childCollectionIndex
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function totalChildTokens(string $collectionIndex, string $tokenId, string $childCollectionIndex)
    {
        if (empty($collectionIndex) || $tokenId == "" || $childCollectionIndex == "") {
            throw new NFTException("collection index, tokenId or childCollection index is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::TotalChildTokens, [$collectionIndex, $tokenId, $childCollectionIndex]);
        return json_decode($respData->getData());
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @param string $childCollectionIndex
     * @param string $index
     * @return mixed
     * @throws NFTException
     * @throws \Dac\Exception\ConfigException
     * @throws \Dac\Exception\DACException
     */
    public function childTokenByIndex(string $collectionIndex, string $tokenId, string $childCollectionIndex, string $index)
    {
        if (empty($collectionIndex) || $tokenId == "" || empty($childCollectionIndex) || $index == "") {
            throw new NFTException("collection index, tokenId, childCollection index or index is empty");
        }
        $respData = $this->dacClient->sendQueryRequest(self::ChildTokenByIndex, [$collectionIndex, $tokenId, $childCollectionIndex, $index]);
        return json_decode($respData->getData());
    }
}