<?php

namespace Dac\Manager;

use Dac\Client\DACClient;
use Dac\Exception\DACException;
use Dac\Exception\NFTException;
use Dac\Model\ID;
use Dac\Model\IdentityInfo;
use Dac\Model\Metadata;
use Dac\Utils\Crypto;
use Dac\Utils\Tools;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use Dac\Model\AuthorizationView;


class ToolManager extends BaseManager
{
    const IdentityRegex = "/^[a-zA-Z0-9]{3,32}$/";
    const ECDSAAlgorithm = "ecdsa_with_sha256";
    const Sha256Hash = "sha256";
    const Ecc256Curve = "prime256v1";

    const Image            = "IMAGE";
    const ImageJPEG        = "JPEG";
    const ImagePNG         = "PNG";
    const ImageGIF         = "GIF";
    const MaxResourceLimit = 5242880;
    const resourceNamePattern = "/^[A-Za-z0-9]{1,57}\\.[A-Za-z0-9]{3,10}$/";

    const BlockStorageSendToAudit = "blockStorageSendToAudit";
    const BlockStorageGetUploadAuthorization = "blockStorageGetUploadAuthorization";

    private array $resourceTypeMap;

    /**
     * @throws DACException
     */
    public function __construct(DACClient $dacClient) {
        parent::__construct($dacClient);
        // 创建ObsClient实例
        $this->resourceTypeMap=array();
        $strings =[];
        $strings[0]=self::ImageJPEG;
        $strings[1]=self::ImagePNG;
        $strings[2]=self::ImageGIF;
        $this->resourceTypeMap[self::Image] = $strings;
    }

    public function newImageMetadata(string $data, string $name, string $url, string $description, int $category)
    {
        if (empty($data)) {
            throw new NFTException("input image is empty");
        }
        if (strlen($name) < 1 || strlen($name) > 100) {
            throw new NFTException("input name invalid, length should in [1,100]");
        }
        if (strlen($url) < 5 || strlen($url) > 400) {
            throw new NFTException("input url invalid, length should in [5,400]");
        }
        $md = new Metadata();
        $md->setType(Metadata::MetaTypeImage);
        $md->setCategory($category);
        $md->setDescription($description);
        $md->setName($name);
        $md->setUrl($url);
        $md->setHash(hash(self::Sha256Hash, $data));
        return $md;
    }

    /**
     * @param string $algorithm
     * @param string $identityName
     * @return IdentityInfo
     * @throws DACException
     */
    public function generateIdentity(string $algorithm, string $identityName)
    {
        if (!preg_match(self::IdentityRegex, $identityName)) {
            throw new DACException("identityName invalid, length should in [3,32]");
        }
        if (strcmp($algorithm, self::ECDSAAlgorithm) != 0) {
            throw new DACException("the sign algorithm should only be ecdsa_with_sha256");
        }
        $newCnf = array("private_key_type" => OPENSSL_KEYTYPE_EC, "curve_name" => self::Ecc256Curve);

        $crsCnf = array('digest_alg' => self::Sha256Hash);
        $exportCnf = array();
        if (!empty($this->dacClient->config->getOpensslCnf())) {
            $newCnf['config'] = $this->dacClient->config->getOpensslCnf();
            $crsCnf['config'] = $this->dacClient->config->getOpensslCnf();
            $exportCnf['config'] = $this->dacClient->config->getOpensslCnf();
        }
        $privkey = openssl_pkey_new($newCnf);
        $caCert = Crypto::readCert($this->dacClient->config->getOrgCaCert());
        $caKey = Crypto::readPrivateKey($this->dacClient->config->getOrgCaKey());

        $dn = array(
            "countryName" => "CN",
            "stateOrProvinceName" => "Beijing",
            "localityName" => "Beijing",
            "organizationName" => Tools::calcAddress($caCert),
            "commonName" => $identityName,
        );

        $csr = openssl_csr_new($dn, $privkey, $crsCnf);

        $userCert = openssl_csr_sign($csr, $caCert, $caKey, 3650, $crsCnf);

        openssl_x509_export($userCert, $certOutString);
        openssl_pkey_export($privkey, $privkeyOutString, null, $exportCnf);

        return new IdentityInfo($certOutString, $privkeyOutString, Tools::calcAddress($userCert));
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return ID
     */
    public function generataID(string $collectionIndex, string $tokenId)
    {
        return new ID($collectionIndex, $tokenId);
    }

    /**
     * @param string $collectionIndex
     * @param string $tokenId
     * @return false|string
     */
    public function generateAssetID(string $collectionIndex, string $tokenId)
    {
        return hash(self::Sha256Hash, $this->generataID($collectionIndex, $tokenId)->getId());
    }

    /**
     * @param string $algorithm
     * @param string $cert
     * @param string $key
     * @return void
     * @throws DACException
     */
    public function switchIdentityWithPem(string $algorithm, string $cert, string $key)
    {
        if (strcmp($algorithm, self::ECDSAAlgorithm) != 0) {
            throw new DACException("the sign algorithm should only be ecdsa_with_sha256");
        }
        $this->dacClient->switchIdentityWithPem($cert, $key);
    }

    /**
     * @param string $identityName
     * @return void
     */
    public function switchIdentityWithName(string $identityName)
    {
        $this->dacClient->switchIdentityWithName($identityName);
    }

    /**
     * @throws DACException
     */
    public function uploadResourceWithAudit(string $resource , string $resourceType, string $resourceContentType, string $resourceName, bool $passReview)
    {
        if (strlen($resource) == 0 || $resourceType == "" || $resourceContentType == "" || $resourceName == "" ){
            throw new DACException("resource , resourceType or resourceContentType  or resourceName string is empty");
        }
        if (strlen($resource) > self::MaxResourceLimit) {
            throw new DACException("the resource is too large");
        }
        if (!$this->validateResourceType($resourceType)) {
            throw new DACException("invalid resourceType");
        }
        if (!$this->validateResourceContentType($resourceType, $resourceContentType)) {
            throw new DACException("invalid  resourceContentType");
        }
        if (!$this->isValidResourceName($resourceName)){
            throw new DACException("invalid resourceName");
        }
        // get authorization
        $request = $this->dacClient->sendResourceSharedRequest(self::BlockStorageGetUploadAuthorization,array($resourceName,$resourceType, $resourceContentType), []);
        $authorizationViewJson = json_decode($request->getBody(), true);

        $signed_url = $authorizationViewJson["signed_url"];
        $request_head = $authorizationViewJson["request_head"];
        $domain = $authorizationViewJson["domain"];
        $resource_path = $authorizationViewJson["resource_path"];
        $resource_url="http://".$domain."/".$resource_path;

        if ($domain ==null){
            throw new DACException("invalid authorization response 'domain'");
        }
        if ($resource_path ==null){
            throw new DACException("invalid authorization response 'resourcePath'");
        }
        // upload
        $this->retryUploadResource($signed_url, $resource, $request_head);
        // audit

        $flag = $this->sendToAudit($resource_path, $resourceType, $resourceContentType,$passReview?"true":"false");
        if (!$flag){
            throw new DACException("sendToAudit no pass result is false");
        }
        $metadata= $this->NewImageMetadata($resource, $resourceName, $resource_url, "", 1);
        return new AuthorizationView($resource_path, $metadata, $resource_url);
    }

    /**
     * @throws DACException
     */
    private function sendToAudit($resource_path, $resourceType, $resourceContentType, $passReview)
    {
        $dacException = null;
        $responseBody = [];
        for ($i = 0; $i < 3; $i++) {
            try {
                $responseBody = $this->dacClient->sendResourceSharedRequest(self::BlockStorageSendToAudit, array($resource_path, $resourceType, $resourceContentType, $passReview), []);
                $this->$dacException = null;
            } catch (DACException $dac) {
                $dacException = $dac;
            }
        }
        if ($dacException != null) {
            throw $dacException;
        }
        $auditJson = json_decode($responseBody->getBody(), true);
        return $auditJson["Result"];
    }
    /**
     * @param string $resourceType
     * @param string $resourceContentType
     * @return bool
     */
    private function  validateResourceContentType(string $resourceType,string $resourceContentType): bool {
        if( $resourceContentType == "") {
            return false;
        }
        $resourceTypes = $this->resourceTypeMap[$resourceType];
        foreach ($resourceTypes as $audioType)
        {
            if( $audioType == strtoupper($resourceContentType)) {
                return true;
            }
        }
        return false;
    }
    /**
     * @param string $resourceType
     * @return bool
     */
    private function  validateResourceType(string $resourceType): bool
    {
        if ($resourceType == "") {
            return false;
        }
    return match ($resourceType) {
        self::Image => true,
        default => false,
    };
    }

    /**
     * @param string $resourceName
     * @return false
     */
    function isValidResourceName(string $resourceName): bool
    {
        if (strlen($resourceName) > 60) {
            return false;
        }
        if (preg_match(self::resourceNamePattern, $resourceName)) {
           return true;
        }
        return false;
    }

    public function retryUploadResource(mixed $signed_url, string $resource, mixed $request_head): void
    {
        $dacException = null;
        for ($i = 0; $i < 3; $i++) {
            try {
                $this->uploadResource($signed_url, $resource, $request_head);
                return;
            } catch (Exception $dac) {
                $this->$dacException = $dac;
            }
        }
        if ($dacException !=null){
            throw $dacException;
        }
    }
    /**
     * @param mixed $signed_url
     * @param string $resource
     * @param mixed $request_head
     * @return void
     * @throws DACException
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function uploadResource(mixed $signed_url, string $resource, mixed $request_head): void
    {
        $httpClient = new Client(['Verify' => false]);

        try {
            $response = $httpClient->request('PUT', $signed_url, ['body' => $resource, 'headers' => $request_head]);
            if ($response->getStatusCode() != 200) {
                throw new DACException("Put object getStatusCode=" + $response->getStatusCode());
            }
            $response->getBody()->close();
        } catch (ClientException $ex) {
            throw new DACException("Put object" + $ex->getMessage());
        }
    }
}