<?php

namespace Dac\Utils;

use Dac\Exception\DACException;
use OpenSSLAsymmetricKey;
use OpenSSLCertificate;

class Crypto
{
    /**
     * @param $keyPath
     * @return false|OpenSSLAsymmetricKey
     * @throws DACException
     */
    public static function readPrivateKey(string $keyPath)
    {
        $content = Tools::readFile($keyPath);
        if (empty($content)) {
            throw new DACException("read key failed");
        }
        return openssl_get_privatekey($content);
    }

    /**
     * @param $keyString
     * @return false|OpenSSLAsymmetricKey
     * @throws DACException
     */
    public static function loadPrivateKey(string $keyString)
    {
        if (empty($keyString)) {
            throw new DACException("load key failed");
        }
        return openssl_get_privatekey($keyString);
    }

    /**
     * @param $certPath
     * @return false|OpenSSLCertificate
     * @throws DACException
     */
    public static function readCert(string $certPath)
    {
        $content = Tools::readFile($certPath);
        if (empty($content)) {
            throw new DACException("read cert failed");
        }
        $r = openssl_x509_read($content);
        if (empty($r)) {
            throw new DACException("read cert error");
        }
        return $r;
    }

    /**
     * @param $certString
     * @return false|OpenSSLCertificate
     * @throws DACException
     */
    public static function loadCert(string $certString)
    {
        if (empty($certString)) {
            throw new DACException("load cert failed");
        }
        $cert = openssl_x509_read($certString);
        if (empty($cert)) {
            throw new DACException("load cert failed");
        }
        return $cert;
    }

    /**
     * @param string $msg
     * @param OpenSSLAsymmetricKey $key
     * @return false
     */
    public static function sign(string $msg, OpenSSLAsymmetricKey $key)
    {
        $signatrue = false;
        openssl_sign($msg, $signatrue, $key, OPENSSL_ALGO_SHA256);
        return $signatrue;
    }

    /**
     * @param $msg
     * @param $sign
     * @param OpenSSLCertificate $cert
     * @return false|int
     */
    public static function verify($msg, $sign, OpenSSLCertificate $cert)
    {
        $publicKey = openssl_get_publickey($cert);
        return openssl_verify($msg, $sign, $publicKey);
    }

    /**
     * @param OpenSSLCertificate $cert
     * @return mixed
     */
    public static function getOrganization(OpenSSLCertificate $cert)
    {
        return openssl_x509_parse($cert)['subject']['O'];
    }

    /**
     * @param OpenSSLCertificate $cert
     * @return mixed
     */
    public static function getCommonName(OpenSSLCertificate $cert)
    {
        return openssl_x509_parse($cert)['subject']['CN'];
    }
}