<?php

namespace Dac\Utils;

use Dac\Exception\DACException;
use Dac\Model\Identity;
use OpenSSLAsymmetricKey;
use OpenSSLCertificate;
use Proto\Common\ContractInvocation;
use Proto\Common\Invocation;
use Proto\Common\RawMessage;
use Proto\Common\RawMessage\Signature;
use Proto\Common\TxHeader;
use Proto\Common\Identity as cIdentity;

class Message
{
    const CommonTransaction = 0;
    const VoteTransaction = 1;
    const CommonName = 0;
    const PlatformVersion = "1.0";
    const Direct = 0;

    public Config $config;

    /**
     * @param Config $config
     */
    public function __construct(Config $config)
    {
        $this->config = $config;
    }

    /**
     * @param string $funcName
     * @param array $args
     * @return RawMessage
     * @throws DACException
     */
    public function buildPreparationMsg(string $funcName, array $args): RawMessage
    {
        $chainID = $this->config->getChainID();
        $identity = $this->config->getIdentity();
        $contractName = $this->config->getContractName();
        $contractInvocation = $this->buildContractInvocation($contractName, $funcName, $args);
        return $this->buildInvokeMessage($chainID, $contractInvocation, $identity, self::Direct);
    }


    /**
     * @param string $chainID
     * @param ContractInvocation $contractInvocation
     * @param Identity $identity
     * @param int $msgType
     * @return RawMessage
     */
    private function buildInvokeMessage(string $chainID, ContractInvocation $contractInvocation, Identity $identity, int $msgType): RawMessage
    {
        $latestBlkNum = 0;
        $domains = ['/**'];
        $invocation = $this->buildInvocation($chainID, $contractInvocation, $identity->getCert(), $latestBlkNum, $domains);
        $payload = $invocation->serializeToString();
        $rawMsg = $this->getRawMessage($payload, $identity->getCertString(), $identity->getKey());
        $rawMsg->setType($msgType);
        return $rawMsg;
    }

    /**
     * @param string $payload
     * @param string $cert
     * @param OpenSSLAsymmetricKey $key
     * @return RawMessage
     */
    private function getRawMessage(string $payload, string $cert, OpenSSLAsymmetricKey $key): RawMessage
    {
        $signature = Crypto::sign($payload, $key);
        $rawMessage = new RawMessage();
        $rawMsgSignature = new Signature();
        $rawMsgSignature->setCert($cert)
            ->setSign($signature);
        $rawMessage->setPayload($payload)
            ->setSignature($rawMsgSignature)
            ->setPlatformVersion(self::PlatformVersion);
        return $rawMessage;
    }


    /**
     * @param string $chainID
     * @param ContractInvocation $contractInvocation
     * @param OpenSSLCertificate $cert
     * @param $latestBlkNum
     * @param array $domains
     * @return Invocation
     */
    private function buildInvocation(string $chainID, ContractInvocation $contractInvocation, OpenSSLCertificate $cert, $latestBlkNum, array $domains): Invocation
    {
        $header = $this->buildTxHeader($chainID, $cert, $latestBlkNum, $domains);
        $invocation = new Invocation();
        $invocation->setHeader($header)
            ->setParameters($contractInvocation);
        return $invocation;
    }

    /**
     * @param string $chainID
     * @param OpenSSLCertificate $cert
     * @param $latestBlkNum
     * @param array $domains
     * @return TxHeader
     */
    private function buildTxHeader(string $chainID, OpenSSLCertificate $cert, $latestBlkNum, array $domains): TxHeader
    {
        $txHeader = new TxHeader();
        $now = Tools::GenerateTimestamp();
        $nonce = Tools::GenerateNonce();
        $txHeader->setType(self::CommonTransaction)
            ->setChainId($chainID)
            ->setCreator($this->buildIdentity($cert))
            ->setTimestamp($now)
            ->setNonce($nonce)
            ->setLatestBlock($latestBlkNum)
            ->setDomains($domains)
            ->setVersion(self::PlatformVersion);
        return $txHeader;
    }

    /**
     * @param OpenSSLCertificate $cert
     * @return cIdentity
     */
    private function buildIdentity(OpenSSLCertificate $cert): cIdentity
    {
        $identity = new cIdentity();
        $identity->setType(self::CommonName)
            ->setOrg(Crypto::getOrganization($cert))
            ->setId(Crypto::getCommonName($cert));
        return $identity;
    }

    /**
     * @param string $contractName
     * @param string $funcName
     * @param array<string> $args
     * @return ContractInvocation
     */
    private function buildContractInvocation(string $contractName, string $funcName, array $args): ContractInvocation
    {
        $contractInvocation = new ContractInvocation();
        $contractInvocation->setContractName($contractName)
            ->setFuncName($funcName)
            ->setArgs($args);
        return $contractInvocation;
    }
}