<?php

namespace Dac\Utils;


use Dac\Client\HttpsClient;
use Dac\Exception\DACException;
use Dac\Exception\NFTException;
use Dac\Model\ResponseData;
use OpenSSLCertificate;
use Proto\Common\CommonTxData;
use Proto\Common\RawMessage;
use Proto\Common\Response;
use Proto\Common\Transaction;
use Proto\Common\TxPayload;


class Tools
{
    const NanoTime = 1000000000;
    const Sha256 = 'sha256';
    const Len256ToLen160Addr = -40;
    const Success = "success";

    /**
     * @throws \Exception
     */
    public static function readFile($filePath)
    {
        if (!file_exists($filePath)) {
            throw new \Exception("file not exist");
        }
        return file_get_contents($filePath);
    }


    public static function strToBytes($string)
    {
        $bytes = array();
        for ($i = 0; $i < strlen($string); $i++) {
            $bytes[] = ord($string[$i]);
        }
        return $bytes;
    }


    public static function GenerateTimestamp()
    {
        return (int)(microtime(true) * self::NanoTime);
    }

    public static function GenerateNonce()
    {
        return rand();
    }

    /**
     * @param RawMessage $rawMessage
     * @return ResponseData|false
     * @throws NFTException
     */
    public static function parseResponseRawData(RawMessage $rawMessage)
    {
        $response = new Response();
        $response->mergeFromString($rawMessage->getPayload());
        if ($response->getStatus() != HttpsClient::OK) {
            throw new NFTException("execute action error, status " . $response->getStatus() . ", info: " . $response->getStatusInfo());
        }
        if (empty($response->getPayload())) {
            return false;
        }
        $transaction = new Transaction();
        $transaction->mergeFromString($response->getPayload());
        $txPayload = new TxPayload();
        $txPayload->mergeFromString($transaction->getPayload());
        $txData = new CommonTxData();
        $txData->mergeFromString($txPayload->getData());
        return new ResponseData(hash(self::Sha256, $transaction->getPayload()), $txData->getResponse()->getPayload());
    }

    /**
     * @param OpenSSLCertificate $cert
     * @return string
     */
    public static function calcAddress(OpenSSLCertificate $cert)
    {
        $pk = openssl_get_publickey($cert);
        $data = openssl_pkey_get_details($pk);
        $h = hash('sha256', asn1DerEncodePk($data['key']));
        return substr($h, self::Len256ToLen160Addr);
    }


    /**
     * @param string $data
     * @return void
     * @throws DACException
     */
    public static function checkResponse(string $data)
    {
        if (strcmp(self::Success, json_decode($data)) != 0) {
            throw new DACException("transaction failed, response data is empty");
        }
    }
}


function asn1DerEncodePk(string $data)
{

    $arr = explode("\n", $data);
    if (count($arr) < 4) {
        throw new DACException("parse public key error");
    }
    $res = "";
    foreach ($arr as $k => $v) {
        if (empty($v) || str_contains($v, 'BEGIN PUBLIC KEY') || str_contains($v, 'END PUBLIC KEY')) {
            continue;
        }
        $res = $res . base64_decode($v);
    }
    return $res;
}
